import { BaseError as CustomError } from 'make-error';
/**
 * Base error class.
 */
export declare class BaseError<A> extends CustomError {
    /**
     * Initializes a new instance of the error.
     *
     * @param message Error message.
     * @param attrs Attributes to merge onto the instance.
     */
    constructor(message?: string, attrs?: Partial<ErrorAttributes<A>>);
    /**
     * Preconfigures an `assert` function.
     *
     * @example
     *   promise.then(MyError.makeAssert('Stuff was not found'))
     */
    static makeAssert<T>(message: string): Asserter<T>;
    /**
     * If `data` is falsy, throws the error with the specified message.
     *
     * @param data The data to check.
     * @param message The message to construct the error with.
     */
    static assert<T>(data: T, message?: string): T | never;
    /**
     * Default toJSON implementation.
     */
    toJSON(): Object;
}
/**
 * Asserter function.
 */
export declare type Asserter<T> = (value: any) => T | never;
/**
 * Error attributes.
 */
export declare type ErrorAttributes<A> = A & {
    [key: string]: any;
};
/**
 * Base error constructor.
 */
export interface BaseErrorConstructor<A> {
    /**
     * Constructs the error.
     */
    new (message?: string, attrs?: Partial<ErrorAttributes<A>>): BaseError<A> & ErrorAttributes<A>;
    /**
     * Makes an asserter function.
     */
    makeAssert<T>(message: string): Asserter<T>;
    /**
     * Asserts the truthiness of the given value, throws the error otherwise.
     */
    assert<T>(data: T, message?: string): T | never;
}
/**
 * Makes a new error class preconfigured with the specified settings,
 * as well as some extra methods for assertions.
 *
 * @param defaultMessage The default error message.
 * @param defaultAttrs The default attributes for creating new instances.
 */
export declare function MakeErrorClass<A = {}>(defaultMessage?: string, defaultAttrs?: A): BaseErrorConstructor<A>;
